window.addEventListener("keydown", _onKeydown);

var _barcode = "";
var _productInfoCallback = null;
var _process = false;
var _videoPlayer = null;
var _currentVideoIndex = 0;
var _videoFilesList = [];

function _onKeydown(e) {
	if (e.key == "Enter" || e.key == "Tab") {
		if (!(_barcode.length === 0)) {
		    PriceChecker.playSound("scan");
			onBarcodeReceived(_barcode);
		}
		_barcode = "";
	} else {
		_barcode += e.key;
	}
}

function toast(str) {
    PriceChecker.toast(str);
}

function setImage(image, base64data, parentWidth, parentHeight) {
    if (base64data === undefined || base64data === null || base64data.length === 0) {
        image.src = null;
        image.width = 0;
        image.height = 0;
        return;
    }

    image.onload = function() {
        var realW = image.naturalWidth;
        var realH = image.naturalHeight;
        var ratio = realW / realH;

        var margin = 20;
        var parentW = parentWidth - margin;
        var parentH = parentHeight - margin;

        var newW = parentW;
        var newH = parentW / ratio;
        if ( newH > parentH ) {
            newH = parentH;
            newW = parentH * ratio;
        }

        image.width = newW;
        image.height = newH;
    };

    image.src = "data:image/jpeg;base64," + base64data;
}

function getProductInfo(requestJson, requestStartCallback, resultFinishCallback) {
    if (_process) {
        PriceChecker.toast(getSettings("language") == "ru" ? "Дождитесь завершения текущей операции!" : "Wait for the current operation to complete!");
        return;
    }

    if ( !(requestStartCallback === undefined || requestStartCallback === null) ) {
        requestStartCallback();
    }

    _process = true;
    _productInfoCallback = resultFinishCallback;
    PriceChecker.getProductInfo(JSON.stringify(requestJson));
}

function onProductInfoReceived(json) {
    _process = false;
    var product = null;
    var badJson = "";

    try {
        product = JSON.parse(json);
    } catch (e) {
        product = { result: -10, message: "Response error: " + e.message };
        badJson = " " + json;
    }

    if ( product.result === undefined || product.message === undefined ) {
        product = { result: -20, message: "Response error: Incompatible response" };
        badJson = " " + json;
    }

    if ( !(product.result === 0) ) {
        PriceChecker.logError(product.message + badJson);
    }

    if ( !(_productInfoCallback === undefined || _productInfoCallback === null) ) {
        _productInfoCallback(product);
    }
}

function logInfo(text) {
    PriceChecker.logInfo(text);
}

function logWarning(text) {
    PriceChecker.logWarning(text);
}

function logError(text) {
    PriceChecker.logError(text);
}

function getSetting(setting) {
    return PriceChecker.getSetting(setting);
}

function _getVideoList() {
    var videoFilesString = PriceChecker.getVideoFiles();
    if (videoFilesString.length === 0) {
        _videoFilesList = [];
        return false;
    }

    _videoFilesList = videoFilesString.split("\n");
    return true;
}

function _playNextVideo() {
    _currentVideoIndex++;

    if ( _currentVideoIndex >= _videoFilesList.length-1 ) {
        _currentVideoIndex = 0;
        if ( !_getVideoList() ) {
            return;
        }
    }

    _videoPlayer.src = _videoFilesList[_currentVideoIndex];
}

function initVideoPlayer(player) {
    if ( !(PriceChecker.getSetting("playVideo") === "true") ) {
        return;
    }

    if (player === undefined || player === null) {
        return;
    }

    _videoPlayer = player;

    if ( !_getVideoList() ) {
        return;
    }

    _videoPlayer.oncanplay = function() {
        _videoPlayer.play();
    };

    _videoPlayer.onended = function() {
        _playNextVideo();
    };

    _videoPlayer.onerror = function() {
        // TODO: delay?

        var errorMessage = _videoFilesList[_currentVideoIndex] + ": ";

        switch (_videoPlayer.error.code) {
            case MediaError.MEDIA_ERR_ABORTED:
              errorMessage += "MEDIA_ERR_ABORTED";
              break;

            case MediaError.MEDIA_ERR_NETWORK:
              errorMessage += "MEDIA_ERR_NETWORK";
              break;

            case MediaError.MEDIA_ERR_DECODE:
              errorMessage += "MEDIA_ERR_DECODE";
              break;

            case MediaError.MEDIA_ERR_SRC_NOT_SUPPORTED:
              errorMessage += "MEDIA_ERR_SRC_NOT_SUPPORTED";
              break;

            default:
              errorMessage += "MEDIA_ERR_UNKNOWN";
              break;
        }

        if ( !(_videoPlayer.error.message === undefined || _videoPlayer.error.message === null) ) {
            errorMessage += ": " + _videoPlayer.error.message;
        }

        PriceChecker.logError(errorMessage);

        _playNextVideo();
    };

    if (PriceChecker.getSetting("playVideoMuted") === "true") {
        _videoPlayer.muted = true;
    }

    _videoPlayer.src = _videoFilesList[_currentVideoIndex];
}

function _videoPause() {
    if (_videoPlayer === null) {
        return;
    }

    if (_videoFilesList.length == 0) {
        return;
    }

    _videoPlayer.pause();
}

function _videoResume() {
    if (_videoPlayer === null) {
        return;
    }

    if (_videoFilesList.length == 0) {
        return;
    }

    _videoPlayer.play();
}

function isWeightBarcode(barcode) {
    if (!(PriceChecker.getSetting("useWeightBarcode") === "true")) {
        return false;
    }

    if ( barcode === undefined || barcode === null ) {
        return false;
    }

    var prefixLen = 2; // always 2
    var prefixes = PriceChecker.getSetting("weightBarcodePrefix").split(",");

    for ( var i = 0; i < prefixes.length; i++ ) {
        if (barcode.substring(0, prefixLen) === prefixes[i]) {
            return true;
        }
    }

    return false;
}

function getWeightBarcodeProductCode(barcode) {
    if ( barcode === undefined || barcode === null ) {
        return "";
    }

    var prefixLen = 2; // always 2
    var productCodeLen = Number(PriceChecker.getSetting("weightBarcodeProductLen"));
    return barcode.substring(prefixLen, prefixLen + productCodeLen);
}

function getWeightBarcodeWeight(barcode) {
    if ( barcode === undefined || barcode === null ) {
        return 0;
    }

    var prefixLen = 2; // always 2
    var productCodeLen = Number(PriceChecker.getSetting("weightBarcodeProductLen"));
    var weightDecimals = Number(PriceChecker.getSetting("weightBarcodeDecimals"));
    var weightLen = 13 - 1 - prefixLen - productCodeLen; // 13 - barcode full len (EAN13), 1 - barcode checksum len

    var weight = Number(barcode.substring(prefixLen + productCodeLen, prefixLen + productCodeLen + weightLen));

    return weight * (1 / Math.pow(10, weightDecimals));
}

/*function getProductPrice(str) {
    if ( str === undefined || str === null || str.length === 0 ) {
        return "";
    }

    var match = str.match(/(\d+(?:[.,]\d+)?)([^\d.,]*)/);
    return match ? match[1] : "";
}

function getProductCurrency(str) {
    if ( str === undefined || str === null || str.length === 0 ) {
        return "";
    }

    var match = str.match(/(\d+(?:[.,]\d+)?)([^\d.,]*)/);
    return match ? match[2] : "";
}*/

function calcWeightProductAmount(price, weight) {
    if ( price === undefined || price === null || price.length === 0 ) {
        return price;
    }

    /*var match = price.match(/(\d+(?:[.,]\d+)?)([^\d.,]*)/);
    if ( !match ) {
        return price;
    }

    var result = (match[1] * weight).toFixed(2).toString();
    var decimal = result.substr(-3);
    if ( decimal === ".00" || decimal === ",00" ) {
        result = result.slice(0, -3);
    }

    result += match[2];*/

    // убираем все лишние символы кроме цифр, точки
    var formattedPrice = price.replace(/[^\d.,]/g, '');

    // вычисляем сумму
    var amount = (formattedPrice * weight).toFixed(2).toString();

    // удаляем .00
    if ( amount.substr(-3) === ".00" ) {
        amount = amount.slice(0, -3);
    }

    return amount;
}
